/*
 *      Copyright (c) 2018-2028, Chill Zhuang All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions are met:
 *
 *  Redistributions of source code must retain the above copyright notice,
 *  this list of conditions and the following disclaimer.
 *  Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in the
 *  documentation and/or other materials provided with the distribution.
 *  Neither the name of the dreamlu.net developer nor the names of its
 *  contributors may be used to endorse or promote products derived from
 *  this software without specific prior written permission.
 *  Author: Chill 庄骞 (smallchill@163.com)
 */
package org.springblade.finance.dto;

import com.fasterxml.jackson.annotation.JsonFormat;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;

import javax.validation.constraints.NotBlank;
import javax.validation.constraints.NotNull;
import java.io.Serializable;
import java.math.BigDecimal;
import java.util.Date;

/**
 * 内部结算-结算产品表实体类
 *
 * @author zbc
 * @since 2022-07-20
 */
@Data
public class InnerComapnySpidDTO implements Serializable {

	private static final long serialVersionUID = 1L;

	@ApiModelProperty(value = "结算单类型")
	@NotBlank(message = "结算单类型不能为空")
	private String dealType;

	@ApiModelProperty(value = "结算方名称")
	@NotBlank(message = "结算方名称不能为空")
	private String company;

	@ApiModelProperty(value = "产品ID")
	@NotBlank(message = "产品ID不能为空")
	private String spid;

	@ApiModelProperty(value = "产品名称")
	@NotBlank(message = "产品名称不能为空")
	private String spname;

	@ApiModelProperty(value = "PRM业务名称")
	@NotBlank(message = "PRM业务名称不能为空")
	private String spnamePrm;

	@ApiModelProperty(value = "产品价格")
	@NotNull(message = "产品价格不能为空")
	private BigDecimal price;

	@ApiModelProperty(value = "结算单价")
	@NotNull(message = "结算单价不能为空")
	private BigDecimal dealPrice;

	@ApiModelProperty(value = "合作开始月份")
	@JsonFormat(pattern = "yyyy-MM-dd")
	@NotNull(message = "合作开始月份不能为空")
	private Date accountStartdate;

	@ApiModelProperty(value = "合作结束月份")
	@JsonFormat(pattern = "yyyy-MM-dd")
	@NotNull(message = "合作结束月份不能为空")
	private Date accountEnddate;

	@ApiModelProperty(value = "是否有效。1：有效；0：无效；")
	@NotNull(message = "是否有效不能为空")
	private Integer isValid;

	@ApiModelProperty(value = "审核人")
	@NotBlank(message = "审核人不能为空")
	private String checker;

}
