/*
 *      Copyright (c) 2018-2028, Chill Zhuang All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions are met:
 *
 *  Redistributions of source code must retain the above copyright notice,
 *  this list of conditions and the following disclaimer.
 *  Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in the
 *  documentation and/or other materials provided with the distribution.
 *  Neither the name of the dreamlu.net developer nor the names of its
 *  contributors may be used to endorse or promote products derived from
 *  this software without specific prior written permission.
 *  Author: Chill 庄骞 (smallchill@163.com)
 */
package org.springblade.finance.entity;

import com.baomidou.mybatisplus.annotation.IdType;
import com.baomidou.mybatisplus.annotation.TableField;
import com.baomidou.mybatisplus.annotation.TableId;
import com.baomidou.mybatisplus.annotation.TableName;
import com.fasterxml.jackson.databind.annotation.JsonSerialize;
import com.fasterxml.jackson.databind.ser.std.ToStringSerializer;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;

import java.io.Serializable;
import java.util.Date;
import java.util.List;

/**
 * 内部结算-结算单审核表实体类
 *
 * @author zbc
 * @since 2022-07-20
 */
@Data
@TableName("INNER_BILL_AUDIT")
public class InnerBillAudit implements Serializable {

	private static final long serialVersionUID = 1L;

	@ApiModelProperty(value = "序号")
	@TableId(value="AID",type = IdType.ASSIGN_ID)
	@JsonSerialize(using= ToStringSerializer.class)
	private Long aid;

	@ApiModelProperty(value = "结算单序号")
	private Long sid;

	@ApiModelProperty(value = "所属结算次数(1-第一次，2-第二次)")
	private Integer audittype;

	@ApiModelProperty(value = "审核状态(1-待审核/2-已推送/3-已驳回/4-待回传/5-已回传/6-已作废)")
	private Integer auditstatus;

	@ApiModelProperty(value = "驳回原因")
	private String rejectreason;

	@ApiModelProperty(value = "创建人ID")
	@JsonSerialize(using = ToStringSerializer.class)
	private Long createbyuserid;

	@ApiModelProperty(value = "创建人姓名")
	private String createbyname;

	@ApiModelProperty(value = "是否报账标记-预结：0：未报账；1：已报账；")
	private Date createtime;

	@ApiModelProperty(value = "审核优先级( 1 - N 依次排序)")
	private Date rejecttime;

	@ApiModelProperty(value = "回传文件名")
	private String uploadfilename;

	@ApiModelProperty(value = "回传文件名下载地址")
	private String uploadfileuri;


	/**
	 * 审核人姓名逗号分隔
	 */
	@TableField(exist = false)
	private String userNames;

	/**
	 * 结算单数据
	 */
	@TableField(exist = false)
	private InnerAccount innerAccount;

	/**
	 * 审核人列表
	 */
	@TableField(exist = false)
	private List<InnerBillAuditUser> userList;



	@TableField(exist = false)
	@ApiModelProperty(value = "内部结算明细列表")
	private List<InnerDealDetail> daelList;

	/**
	 * 回传文件ID
	 */
	@TableField(exist = false)
	private String fid;


	@ApiModelProperty(value = "回传时间[后台设置]")
	@TableField(exist = false)
	private Date uploadTime;


}
